@@ -1,6 +1,6 @@
-/* $Id: os_core_symbian.cpp 1269 2007-05-12 15:03:23Z bennylp $ */
 /* 
- * Copyright (C)2003-2006 Benny Prijono <benny@prijono.org>
+ * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
+ * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,11 +30,18 @@
 #include "os_symbian.h"
 
 
-#define PJ_MAX_TLS	    32
-#define DUMMY_MUTEX	    ((pj_mutex_t*)101)
-#define DUMMY_SEMAPHORE	    ((pj_sem_t*)102)
-#define THIS_FILE	    "os_core_symbian.c"
- 
+#define PJ_MAX_TLS          32
+#define DUMMY_MUTEX         ((pj_mutex_t*)101)
+#define DUMMY_SEMAPHORE     ((pj_sem_t*)102)
+#define THIS_FILE           "os_core_symbian.c"
+
+/* Default message slot number for RSocketServ::Connect().
+ * Increase it to 32 from the default 8 (KESockDefaultMessageSlots)
+ */
+#ifndef PJ_SYMBIAN_SOCK_MSG_SLOTS
+#  define PJ_SYMBIAN_SOCK_MSG_SLOTS  32
+#endif
+
 /*
  * Note:
  *
@@ -43,22 +50,22 @@
 
 struct pj_thread_t
 {
-    char	    obj_name[PJ_MAX_OBJ_NAME];
-    void	   *tls_values[PJ_MAX_TLS];
+    char            obj_name[PJ_MAX_OBJ_NAME];
+    void           *tls_values[PJ_MAX_TLS];
 
 #if defined(PJ_OS_HAS_CHECK_STACK) && PJ_OS_HAS_CHECK_STACK!=0
-    pj_uint32_t	    stk_size;
-    pj_uint32_t	    stk_max_usage;
-    char	   *stk_start;
-    const char	   *caller_file;
-    int		    caller_line;
+    pj_uint32_t     stk_size;
+    pj_uint32_t     stk_max_usage;
+    char           *stk_start;
+    const char     *caller_file;
+    int             caller_line;
 #endif
 
 } main_thread;
 
 struct pj_atomic_t
 {
-    pj_atomic_value_t	value;
+    pj_atomic_value_t   value;
 };
 
 struct pj_sem_t
@@ -67,6 +74,9 @@ struct pj_sem_t
     int max;
 };
 
+/* Flag and reference counter for PJLIB instance */
+static int initialized;
+
 /* Flags to indicate which TLS variables have been used */
 static int tls_vars[PJ_MAX_TLS];
 
@@ -75,8 +85,6 @@ static unsigned atexit_count;
 static void (*atexit_func[32])(void);
 
 
-
-
 /////////////////////////////////////////////////////////////////////////////
 //
 // CPjTimeoutTimer implementation
@@ -89,8 +97,7 @@ CPjTimeoutTimer::CPjTimeoutTimer()
 
 CPjTimeoutTimer::~CPjTimeoutTimer()
 {
-    if (IsActive())
-	Cancel();
+    Cancel();
     timer_.Close();
 }
 
@@ -115,8 +122,7 @@ CPjTimeoutTimer *CPjTimeoutTimer::NewL()
 
 void CPjTimeoutTimer::StartTimer(TUint miliSeconds)
 {
-    if (IsActive())
-	Cancel();
+    Cancel();
 
     hasTimedOut_ = PJ_FALSE;
     timer_.After(iStatus, miliSeconds * 1000);
@@ -152,11 +158,23 @@ TInt CPjTimeoutTimer::RunError(TInt aError)
 //
 
 PjSymbianOS::PjSymbianOS()
-: isSocketServInitialized_(false), isResolverInitialized_(false),
-  console_(NULL), selectTimeoutTimer_(NULL)
+: isConnectionUp_(false),
+  isSocketServInitialized_(false), isResolverInitialized_(false),
+  console_(NULL), selectTimeoutTimer_(NULL),
+  appSocketServ_(NULL), appConnection_(NULL), appHostResolver_(NULL),
+  appHostResolver6_(NULL)
 {
 }
 
+// Set parameters
+void PjSymbianOS::SetParameters(pj_symbianos_params *params) 
+{
+    appSocketServ_ = (RSocketServ*) params->rsocketserv;
+    appConnection_ = (RConnection*) params->rconnection;
+    appHostResolver_ = (RHostResolver*) params->rhostresolver;
+    appHostResolver6_ = (RHostResolver*) params->rhostresolver6;
+}
+
 // Get PjSymbianOS instance
 PjSymbianOS *PjSymbianOS::Instance()
 {
@@ -175,26 +193,52 @@ TInt PjSymbianOS::Initialize()
 #if 0
     pj_assert(console_ == NULL);
     TRAPD(err, console_ = Console::NewL(_L("PJLIB"), 
-				        TSize(KConsFullScreen,KConsFullScreen)));
+                                        TSize(KConsFullScreen,KConsFullScreen)));
     return err;
 #endif
 
-    if (!isSocketServInitialized_) {
-	err = socketServ_.Connect();
-	if (err != KErrNone)
-	    goto on_error;
+    /* Only create RSocketServ if application doesn't specify it
+     * in the parameters
+     */
+    if (!isSocketServInitialized_ && appSocketServ_ == NULL) {
+        err = socketServ_.Connect(PJ_SYMBIAN_SOCK_MSG_SLOTS);
+        if (err != KErrNone)
+            goto on_error;
 
-	isSocketServInitialized_ = true;
+        isSocketServInitialized_ = true;
     }
 
     if (!isResolverInitialized_) {
-	err = hostResolver_.Open(SocketServ(), KAfInet, KSockStream);
-	if (err != KErrNone)
-	    goto on_error;
-
-	isResolverInitialized_ = true;
+        if (appHostResolver_ == NULL) {
+            if (Connection())
+                err = hostResolver_.Open(SocketServ(), KAfInet, KSockStream,
+                                         *Connection());
+            else
+                err = hostResolver_.Open(SocketServ(), KAfInet, KSockStream);
+        
+            if (err != KErrNone)
+                goto on_error;
+        }
+        
+#if defined(PJ_HAS_IPV6) && PJ_HAS_IPV6!=0
+        if (appHostResolver6_ == NULL) {
+            if (Connection())
+                err = hostResolver6_.Open(SocketServ(), KAfInet6, KSockStream,
+                                          *Connection());
+            else
+                err = hostResolver6_.Open(SocketServ(), KAfInet6, KSockStream);
+        
+            if (err != KErrNone)
+                goto on_error;
+        }
+#endif
+        
+        
+        isResolverInitialized_ = true;
     }
 
+    isConnectionUp_ = true;
+    
     return KErrNone;
 
 on_error:
@@ -205,25 +249,31 @@ on_error:
 // Shutdown
 void PjSymbianOS::Shutdown()
 {
+    isConnectionUp_ = false;
+    
     if (isResolverInitialized_) {
-	hostResolver_.Close();
-	isResolverInitialized_ = false;
+                hostResolver_.Close();
+#if defined(PJ_HAS_IPV6) && PJ_HAS_IPV6!=0
+        hostResolver6_.Close();
+#endif
+        isResolverInitialized_ = false;
     }
 
     if (isSocketServInitialized_) {
-	socketServ_.Close();
-	isSocketServInitialized_ = false;
+        socketServ_.Close();
+        isSocketServInitialized_ = false;
     }
 
-    if (console_) {
-	delete console_;
-	console_ = NULL;
-    }
+    delete console_;
+    console_ = NULL;
 
-    if (selectTimeoutTimer_) {
-	delete selectTimeoutTimer_;
-	selectTimeoutTimer_ = NULL;
-    }
+    delete selectTimeoutTimer_;
+    selectTimeoutTimer_ = NULL;
+    
+    appSocketServ_ = NULL;
+    appConnection_ = NULL;
+    appHostResolver_ = NULL;
+    appHostResolver6_ = NULL;
 }
 
 // Convert to Unicode
@@ -260,7 +310,21 @@ PJ_DEF(pj_uint32_t) pj_getpid(void)
 }
 
 
-PJ_DECL(void) pj_shutdown(void);
+/* Set Symbian specific parameters */
+PJ_DEF(pj_status_t) pj_symbianos_set_params(pj_symbianos_params *prm) 
+{
+    PJ_ASSERT_RETURN(prm != NULL, PJ_EINVAL);
+    PjSymbianOS::Instance()->SetParameters(prm);
+    return PJ_SUCCESS;
+}
+
+
+/* Set connection status */
+PJ_DEF(void) pj_symbianos_set_connection_status(pj_bool_t up)
+{
+    PjSymbianOS::Instance()->SetConnectionStatus(up != 0);
+}
+
 
 /*
  * pj_init(void).
@@ -268,8 +332,15 @@ PJ_DECL(void) pj_shutdown(void);
  */
 PJ_DEF(pj_status_t) pj_init(void)
 {
+        char stack_ptr;
     pj_status_t status;
     
+    /* Check if PJLIB have been initialized */
+    if (initialized) {
+        ++initialized;
+        return PJ_SUCCESS;
+    }
+
     pj_ansi_strcpy(main_thread.obj_name, "pjthread");
 
     // Init main thread
@@ -283,14 +354,29 @@ PJ_DEF(pj_status_t) pj_init(void)
     TInt err; 
     err = os->Initialize();
     if (err != KErrNone)
-	goto on_error;
-    
+        return PJ_RETURN_OS_ERROR(err);
+
+    /* Init logging */
+    pj_log_init();
+
     /* Initialize exception ID for the pool. 
      * Must do so after critical section is configured.
      */ 
     status = pj_exception_id_alloc("PJLIB/No memory", &PJ_NO_MEMORY_EXCEPTION);
     if (status != PJ_SUCCESS)
-        return status;
+        goto on_error;
+
+#if defined(PJ_OS_HAS_CHECK_STACK) && PJ_OS_HAS_CHECK_STACK!=0
+    main_thread.stk_start = &stack_ptr;
+    main_thread.stk_size = 0xFFFFFFFFUL;
+    main_thread.stk_max_usage = 0;
+#else
+    stack_ptr = '\0';
+#endif
+
+    /* Flag PJLIB as initialized */
+    ++initialized;
+    pj_assert(initialized == 1);
 
     PJ_LOG(5,(THIS_FILE, "PJLIB initialized."));
     return PJ_SUCCESS;
@@ -304,7 +390,7 @@ on_error:
 PJ_DEF(pj_status_t) pj_atexit(pj_exit_callback func)
 {
     if (atexit_count >= PJ_ARRAY_SIZE(atexit_func))
-	return PJ_ETOOMANY;
+        return PJ_ETOOMANY;
 
     atexit_func[atexit_count++] = func;
     return PJ_SUCCESS;
@@ -314,16 +400,21 @@ PJ_DEF(pj_status_t) pj_atexit(pj_exit_callback func)
 
 PJ_DEF(void) pj_shutdown(void)
 {
+    /* Only perform shutdown operation when 'initialized' reaches zero */
+    pj_assert(initialized > 0);
+    if (--initialized != 0)
+        return;
+
     /* Call atexit() functions */
     while (atexit_count > 0) {
-	(*atexit_func[atexit_count-1])();
-	--atexit_count;
+        (*atexit_func[atexit_count-1])();
+        --atexit_count;
     }
 
     /* Free exception ID */
     if (PJ_NO_MEMORY_EXCEPTION != -1) {
-	pj_exception_id_free(PJ_NO_MEMORY_EXCEPTION);
-	PJ_NO_MEMORY_EXCEPTION = -1;
+        pj_exception_id_free(PJ_NO_MEMORY_EXCEPTION);
+        PJ_NO_MEMORY_EXCEPTION = -1;
     }
 
     /* Clear static variables */
@@ -333,12 +424,158 @@ PJ_DEF(void) pj_shutdown(void)
     os->Shutdown();
 }
 
+/////////////////////////////////////////////////////////////////////////////
+
+class CPollTimeoutTimer : public CActive 
+{
+public:
+    static CPollTimeoutTimer* NewL(int msec, TInt prio);
+    ~CPollTimeoutTimer();
+    
+    virtual void RunL();
+    virtual void DoCancel();
+
+private:        
+    RTimer           rtimer_;
+    
+    explicit CPollTimeoutTimer(TInt prio);
+    void ConstructL(int msec);
+};
+
+CPollTimeoutTimer::CPollTimeoutTimer(TInt prio)
+: CActive(prio)
+{
+}
+
+
+CPollTimeoutTimer::~CPollTimeoutTimer() 
+{
+    rtimer_.Close();
+}
+
+void CPollTimeoutTimer::ConstructL(int msec) 
+{
+    rtimer_.CreateLocal();
+    CActiveScheduler::Add(this);
+    rtimer_.After(iStatus, msec*1000);
+    SetActive();
+}
+
+CPollTimeoutTimer* CPollTimeoutTimer::NewL(int msec, TInt prio) 
+{
+    CPollTimeoutTimer *self = new CPollTimeoutTimer(prio);
+    CleanupStack::PushL(self);
+    self->ConstructL(msec);    
+    CleanupStack::Pop(self);
+
+    return self;
+}
+
+void CPollTimeoutTimer::RunL() 
+{
+}
+
+void CPollTimeoutTimer::DoCancel() 
+{
+     rtimer_.Cancel();
+}
+
+
+/*
+ * Wait the completion of any Symbian active objects. 
+ */
+PJ_DEF(pj_bool_t) pj_symbianos_poll(int priority, int ms_timeout)
+{
+    CPollTimeoutTimer *timer = NULL;
+    
+    if (priority==-1)
+        priority = EPriorityNull;
+    
+    if (ms_timeout >= 0) {
+        timer = CPollTimeoutTimer::NewL(ms_timeout, priority);
+    }
+    
+    PjSymbianOS::Instance()->WaitForActiveObjects(priority);
+    
+    if (timer) {
+        bool timer_is_active = timer->IsActive();
+    
+        timer->Cancel();
+        
+        delete timer;
+        
+        return timer_is_active ? PJ_TRUE : PJ_FALSE;
+        
+    } else {
+        return PJ_TRUE;
+    }
+}
+
+
+/*
+ * pj_thread_is_registered()
+ */
+PJ_DEF(pj_bool_t) pj_thread_is_registered(void)
+{
+    return PJ_FALSE;
+}
+
+
+/*
+ * Get thread priority value for the thread.
+ */
+PJ_DEF(int) pj_thread_get_prio(pj_thread_t *thread)
+{
+    PJ_UNUSED_ARG(thread);
+    return 1;
+}
+
+
+/*
+ * Set the thread priority.
+ */
+PJ_DEF(pj_status_t) pj_thread_set_prio(pj_thread_t *thread,  int prio)
+{
+    PJ_UNUSED_ARG(thread);
+    PJ_UNUSED_ARG(prio);
+    return PJ_SUCCESS;
+}
+
+
+/*
+ * Get the lowest priority value available on this system.
+ */
+PJ_DEF(int) pj_thread_get_prio_min(pj_thread_t *thread)
+{
+    PJ_UNUSED_ARG(thread);
+    return 1;
+}
+
+
+/*
+ * Get the highest priority value available on this system.
+ */
+PJ_DEF(int) pj_thread_get_prio_max(pj_thread_t *thread)
+{
+    PJ_UNUSED_ARG(thread);
+    return 1;
+}
+
+
+/*
+ * pj_thread_get_os_handle()
+ */
+PJ_DEF(void*) pj_thread_get_os_handle(pj_thread_t *thread) 
+{
+    PJ_UNUSED_ARG(thread);
+    return NULL;
+}
 
 /*
  * pj_thread_register(..)
  */
 PJ_DEF(pj_status_t) pj_thread_register ( const char *cstr_thread_name,
-					 pj_thread_desc desc,
+                                         pj_thread_desc desc,
                                          pj_thread_t **thread_ptr)
 {
     PJ_UNUSED_ARG(cstr_thread_name);
@@ -352,12 +589,12 @@ PJ_DEF(pj_status_t) pj_thread_register ( const char *cstr_thread_name,
  * pj_thread_create(...)
  */
 PJ_DEF(pj_status_t) pj_thread_create( pj_pool_t *pool, 
-				      const char *thread_name,
-				      pj_thread_proc *proc, 
-				      void *arg,
-				      pj_size_t stack_size, 
-				      unsigned flags,
-				      pj_thread_t **ptr_thread)
+                                      const char *thread_name,
+                                      pj_thread_proc *proc, 
+                                      void *arg,
+                                      pj_size_t stack_size, 
+                                      unsigned flags,
+                                      pj_thread_t **ptr_thread)
 {
     PJ_UNUSED_ARG(pool);
     PJ_UNUSED_ARG(thread_name);
@@ -421,11 +658,7 @@ PJ_DEF(pj_status_t) pj_thread_destroy(pj_thread_t *rec)
 PJ_DEF(pj_status_t) pj_thread_sleep(unsigned msec)
 {
     User::After(msec*1000);
-    
-    TInt aError;
-    while (CActiveScheduler::RunIfReady(aError, EPriorityMuchLess))
-    	;
-    
+
     return PJ_SUCCESS;
 }
 
@@ -441,12 +674,12 @@ PJ_DEF(pj_status_t) pj_thread_local_alloc(long *index)
 
     /* Find unused TLS variable */
     for (i=0; i<PJ_ARRAY_SIZE(tls_vars); ++i) {
-	if (tls_vars[i] == 0)
-	    break;
+        if (tls_vars[i] == 0)
+            break;
     }
 
     if (i == PJ_ARRAY_SIZE(tls_vars))
-	return PJ_ETOOMANY;
+        return PJ_ETOOMANY;
 
     tls_vars[i] = 1;
     *index = i;
@@ -460,7 +693,7 @@ PJ_DEF(pj_status_t) pj_thread_local_alloc(long *index)
 PJ_DEF(void) pj_thread_local_free(long index)
 {
     PJ_ASSERT_ON_FAIL(index >= 0 && index < (int)PJ_ARRAY_SIZE(tls_vars) &&
-		     tls_vars[index] != 0, return);
+                     tls_vars[index] != 0, return);
 
     tls_vars[index] = 0;
 }
@@ -474,7 +707,7 @@ PJ_DEF(pj_status_t) pj_thread_local_set(long index, void *value)
     pj_thread_t *rec = pj_thread_this();
 
     PJ_ASSERT_RETURN(index >= 0 && index < (int)PJ_ARRAY_SIZE(tls_vars) &&
-		     tls_vars[index] != 0, PJ_EINVAL);
+                     tls_vars[index] != 0, PJ_EINVAL);
 
     rec->tls_values[index] = value;
     return PJ_SUCCESS;
@@ -488,7 +721,7 @@ PJ_DEF(void*) pj_thread_local_get(long index)
     pj_thread_t *rec = pj_thread_this();
 
     PJ_ASSERT_RETURN(index >= 0 && index < (int)PJ_ARRAY_SIZE(tls_vars) &&
-		     tls_vars[index] != 0, NULL);
+                     tls_vars[index] != 0, NULL);
 
     return rec->tls_values[index];
 }
@@ -499,8 +732,8 @@ PJ_DEF(void*) pj_thread_local_get(long index)
  * Create atomic variable.
  */
 PJ_DEF(pj_status_t) pj_atomic_create( pj_pool_t *pool, 
-				      pj_atomic_value_t initial,
-				      pj_atomic_t **atomic )
+                                      pj_atomic_value_t initial,
+                                      pj_atomic_t **atomic )
 {
     *atomic = (pj_atomic_t*)pj_pool_alloc(pool, sizeof(struct pj_atomic_t));
     (*atomic)->value = initial;
@@ -522,7 +755,7 @@ PJ_DEF(pj_status_t) pj_atomic_destroy( pj_atomic_t *atomic_var )
  * Set the value of an atomic type, and return the previous value.
  */
 PJ_DEF(void) pj_atomic_set( pj_atomic_t *atomic_var, 
-			    pj_atomic_value_t value)
+                            pj_atomic_value_t value)
 {
     atomic_var->value = value;
 }
@@ -561,7 +794,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_inc_and_get(pj_atomic_t *atomic_var)
 PJ_DEF(void) pj_atomic_dec(pj_atomic_t *atomic_var)
 {
     --atomic_var->value;
-}	
+}       
 
 
 /*
@@ -577,7 +810,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_dec_and_get(pj_atomic_t *atomic_var)
  * Add a value to an atomic type.
  */
 PJ_DEF(void) pj_atomic_add( pj_atomic_t *atomic_var,
-			    pj_atomic_value_t value)
+                            pj_atomic_value_t value)
 {
     atomic_var->value += value;
 }
@@ -587,7 +820,7 @@ PJ_DEF(void) pj_atomic_add( pj_atomic_t *atomic_var,
  * Add a value to an atomic type and get the result.
  */
 PJ_DEF(pj_atomic_value_t) pj_atomic_add_and_get( pj_atomic_t *atomic_var,
-			                         pj_atomic_value_t value)
+                                                 pj_atomic_value_t value)
 {
     atomic_var->value += value;
     return atomic_var->value;
@@ -599,7 +832,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_add_and_get( pj_atomic_t *atomic_var,
 
 PJ_DEF(pj_status_t) pj_mutex_create( pj_pool_t *pool, 
                                      const char *name,
-				     int type, 
+                                     int type, 
                                      pj_mutex_t **mutex)
 {
     PJ_UNUSED_ARG(pool);
@@ -615,15 +848,15 @@ PJ_DEF(pj_status_t) pj_mutex_create( pj_pool_t *pool,
  */
 PJ_DEF(pj_status_t) pj_mutex_create_simple( pj_pool_t *pool, 
                                             const char *name,
-					    pj_mutex_t **mutex )
+                                            pj_mutex_t **mutex )
 {
     return pj_mutex_create(pool, name, PJ_MUTEX_SIMPLE, mutex);
 }
 
 
 PJ_DEF(pj_status_t) pj_mutex_create_recursive( pj_pool_t *pool,
-					       const char *name,
-					       pj_mutex_t **mutex )
+                                               const char *name,
+                                               pj_mutex_t **mutex )
 {
     return pj_mutex_create(pool, name, PJ_MUTEX_RECURSE, mutex);
 }
@@ -700,9 +933,9 @@ PJ_DEF(void) pj_leave_critical_section(void)
  */
 PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool, 
                                    const char *name,
-				   unsigned initial, 
+                                   unsigned initial, 
                                    unsigned max,
-				   pj_sem_t **p_sem)
+                                   pj_sem_t **p_sem)
 {
     pj_sem_t *sem;
  
@@ -724,11 +957,11 @@ PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool,
 PJ_DEF(pj_status_t) pj_sem_wait(pj_sem_t *sem)
 {
     if (sem->value > 0) {
-	sem->value--;
-	return PJ_SUCCESS;
+        sem->value--;
+        return PJ_SUCCESS;
     } else {
-	pj_assert(!"Unexpected!");
-	return PJ_EINVALIDOP;
+        pj_assert(!"Unexpected!");
+        return PJ_EINVALIDOP;
     }
 }
 
@@ -739,11 +972,11 @@ PJ_DEF(pj_status_t) pj_sem_wait(pj_sem_t *sem)
 PJ_DEF(pj_status_t) pj_sem_trywait(pj_sem_t *sem)
 {
     if (sem->value > 0) {
-	sem->value--;
-	return PJ_SUCCESS;
+        sem->value--;
+        return PJ_SUCCESS;
     } else {
-	pj_assert(!"Unexpected!");
-	return PJ_EINVALIDOP;
+        pj_assert(!"Unexpected!");
+        return PJ_EINVALIDOP;
     }
 }
 
@@ -782,16 +1015,16 @@ PJ_DEF(void) pj_thread_check_stack(const char *file, int line)
 
     /* Calculate current usage. */
     usage = (&stk_ptr > thread->stk_start) ? &stk_ptr - thread->stk_start :
-		thread->stk_start - &stk_ptr;
+                thread->stk_start - &stk_ptr;
 
     /* Assert if stack usage is dangerously high. */
     pj_assert("STACK OVERFLOW!! " && (usage <= thread->stk_size - 128));
 
     /* Keep statistic. */
     if (usage > thread->stk_max_usage) {
-	thread->stk_max_usage = usage;
-	thread->caller_file = file;
-	thread->caller_line = line;
+        thread->stk_max_usage = usage;
+        thread->caller_file = file;
+        thread->caller_line = line;
     }
 }
 
@@ -807,8 +1040,8 @@ PJ_DEF(pj_uint32_t) pj_thread_get_stack_max_usage(pj_thread_t *thread)
  * Dump thread stack status. 
  */
 PJ_DEF(pj_status_t) pj_thread_get_stack_info(pj_thread_t *thread,
-					     const char **file,
-					     int *line)
+                                             const char **file,
+                                             int *line)
 {
     pj_assert(thread);
 
@@ -817,4 +1050,13 @@ PJ_DEF(pj_status_t) pj_thread_get_stack_info(pj_thread_t *thread,
     return 0;
 }
 
-#endif	/* PJ_OS_HAS_CHECK_STACK */
+#endif  /* PJ_OS_HAS_CHECK_STACK */
+
+/*
+ * pj_run_app()
+ */
+PJ_DEF(int) pj_run_app(pj_main_func_ptr main_func, int argc, char *argv[],
+                       unsigned flags)
+{
+    return (*main_func)(argc, argv);
+}
