@@ -1,4 +1,3 @@
-/* $Id$ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -106,7 +105,9 @@ int errno_test(void)
     /*
      * Unix errors
      */
-#   if defined(EINVAL) && !defined(PJ_SYMBIAN)
+#   if defined(EINVAL) && !defined(PJ_SYMBIAN) && !defined(PJ_WIN32) \
+       && !defined(PJ_WIN64)
+
     rc = PJ_STATUS_FROM_OS(EINVAL);
     pj_set_os_error(rc);
 
@@ -167,6 +168,6 @@ int errno_test(void)
 }
 
 
-#endif	/* INCLUDE_ERRNO_TEST */
+#endif  /* INCLUDE_ERRNO_TEST */
 
 
